#
# Utility functions for writing out gateway C++ files
#
# make_interface(name, method_names)
#   Write out a .cpp and a .h file (into the current directory) for an
#   interface with the given name and given list of method names.
#

import regsub


def make_interface(name, method_names):
  _write_ifc_h(name, method_names)
  _write_ifc_cpp(name, method_names)

def make_gateway(name, method_names):
  _write_gw_h(name, method_names)
  _write_gw_cpp(name, method_names)


###########################################################################
#
# INTERNAL FUNCTIONS
#
#

def _write_ifc_h(name, method_names):
  f = open('Py%s.h' % name, 'w')
  f.write('''\
class Py%s : public PyIUnknown
{
public:
	MAKE_PYCOM_CTOR(Py%s);
	static %s *GetI(PyObject *self);
	static PyComTypeObject type;

	// The Python methods
''' % (name, name, name))
  for method in method_names:
    f.write('\tstatic PyObject *%s(PyObject *self, PyObject *args);\n' % method)
  f.write('''\

protected:
	Py%s(IUnknown *pdisp);
	~Py%s();
};
''' % (name, name))
  f.close()

def _write_ifc_cpp(name, method_names):
  f = open('Py%s.cpp' % name, 'w')
  f.write('''\
#include "stdafx.h"
#include "PythonCOM.h"
#include "Py%s.h"

Py%s::Py%s(IUnknown *pdisp):
	PyIUnknown(pdisp)
{
	ob_type = &type;
}

Py%s::~Py%s()
{
}

/* static */ %s *Py%s::GetI(PyObject *self)
{
	return (%s *)PyIUnknown::GetI(self);
}

''' % (name, name, name, name, name, name, name, name))

  ptr = regsub.gsub('[a-z]', '', name)
  for method in method_names:
    f.write('''\
/* static */ PyObject *Py%s::%s(PyObject *self, PyObject *args)
{
	// ### add the arguments
	if ( !PyArg_ParseTuple(args, ":%s") )
		return NULL;

	%s *p%s = GetI(self);
	if ( p%s == NULL )
		return NULL;

	// ### coerce arguments to C++ types

	HRESULT hr = p%s->%s(/* ### pass args */);
	if ( FAILED(hr) )
		return OleSetOleError(hr);

	// ### coerce result to Python types

	Py_INCREF(Py_None);
	return Py_None;
}

''' % (name, method, method, name, ptr, ptr, ptr, method))

  f.write('static struct PyMethodDef Py%s_methods[] =\n{\n' % name)
  for method in method_names:
    f.write('\t{ "%s", Py%s::%s, 1 },\n' % (method, name, method))

  f.write('''\
	{ NULL }
};

PyComTypeObject Py%s::type("Py%s",
		&PyIUnknown::type,
		sizeof(Py%s),
		Py%s_methods,
		GET_PYCOM_CTOR(Py%s));
''' % (name, name, name, name, name))
  f.close()


def _write_gw_h(name, method_names):
  gname = 'PyG' + name[1:]
  f = open('%s.h' % gname, 'w')
  f.write('''\
class %s : public PyGatewayBase, public %s
{
protected:
	%s(PyObject *instance) : PyGatewayBase(instance) { ; }
	PYGATEWAY_MAKE_SUPPORT(%s, %s, IID_%s)

	// IUnknown and IDispatch
	STDMETHOD_(ULONG,AddRef)(void);
	STDMETHOD_(ULONG,Release)(void);
	STDMETHOD(QueryInterface)(REFIID iid, void ** obj);
	STDMETHOD(GetTypeInfoCount)(UINT FAR* pctInfo);
	STDMETHOD(GetTypeInfo)(UINT itinfo, LCID lcid, ITypeInfo FAR* FAR* pptInfo);
	STDMETHOD(GetIDsOfNames)(REFIID refiid,	OLECHAR FAR* FAR* rgszNames, UINT cNames, LCID lcid, DISPID FAR* rgdispid);
	STDMETHOD(Invoke)(DISPID dispid, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS FAR* params, VARIANT FAR* pVarResult, EXCEPINFO FAR* pexcepinfo, UINT FAR* puArgErr);

	// %s
''' % (gname, name, gname, gname, name, name, name))

  for method in method_names:
    f.write('\tSTDMETHOD(%s)(\n\t\tvoid);\n\n' % method)

  f.write('};\n')
  f.close()

def _write_gw_cpp(name, method_names):
  gname = 'PyG' + name[1:]
  f = open('%s.cpp' % gname, 'w')
  f.write('''\
#include "stdafx.h"
#include "%s.h"

// Std delegation
STDMETHODIMP_(ULONG) %s::AddRef(void) {return PyGatewayBase::AddRef();}
STDMETHODIMP_(ULONG) %s::Release(void) {return PyGatewayBase::Release();}
STDMETHODIMP %s::QueryInterface(REFIID iid, void ** obj) {return PyGatewayBase::QueryInterface(iid, obj);}
STDMETHODIMP %s::GetTypeInfoCount(UINT FAR* pctInfo) {return PyGatewayBase::GetTypeInfoCount(pctInfo);}
STDMETHODIMP %s::GetTypeInfo(UINT itinfo, LCID lcid, ITypeInfo FAR* FAR* pptInfo) {return PyGatewayBase::GetTypeInfo(itinfo, lcid, pptInfo);}
STDMETHODIMP %s::GetIDsOfNames(REFIID refiid, OLECHAR FAR* FAR* rgszNames, UINT cNames, LCID lcid, DISPID FAR* rgdispid) {return PyGatewayBase::GetIDsOfNames( refiid, rgszNames, cNames, lcid, rgdispid);}
STDMETHODIMP %s::Invoke(DISPID dispid, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS FAR* params, VARIANT FAR* pVarResult, EXCEPINFO FAR* pexcepinfo, UINT FAR* puArgErr) {return PyGatewayBase::Invoke( dispid, riid, lcid, wFlags, params, pVarResult, pexcepinfo, puArgErr);}

''' % (gname, gname, gname, gname, gname, gname, gname, gname))

  for method in method_names:
    f.write('''\
STDMETHODIMP %s::%s(
		void)
{
	// ### validate args and perform coercion
	PyObject *result = DispatchViaPolicy("%s", NULL);
	if ( result == NULL )
		return PyCom_HandlePythonFailureToCOM();

	// ### coerce to C++ types and return

	Py_DECREF(result);
	return S_OK;
}

''' % (gname, method, method))

  f.close()